@@ -3,7 +3,9 @@ use warnings;
 package C::Blocks::StretchyBuffer;
 use C::Blocks;
 use C::Blocks::PerlAPI;
-our $VERSION = '0.02_01';
+our $VERSION = '0.05_02';
+$VERSION = eval $VERSION;
+no warnings qw(C::Blocks::compiler);
 
 cshare {
 	#define sbfree(a)         ((a) ? Safefree(stb__sbraw(a)),0 : 0)
